//! A single element of text.
use super::combinator::context_many1;
use super::error::Res;
use super::paragraph::paragraph;
use super::parser_context::ContextElement;
use super::parser_context::ContextTree;
use super::text::Paragraph;
use super::token::Token;
use super::Context;
use nom::IResult;

type UnboundMatcher<'r, 's, I, O, E> = dyn Fn(Context<'r, 's>, I) -> IResult<I, O, E>;

pub fn document(input: &str) -> Res<&str, Vec<Paragraph>> {
    let initial_context: ContextTree<'_, '_> = ContextTree::new();
    let (remaining, tokens) = context_many1(&initial_context, paragraph)(input)?;
    let paragraphs = tokens
        .into_iter()
        .map(|token| match token {
            Token::TextElement(_) => unreachable!(),
            Token::Paragraph(paragraph) => paragraph,
        })
        .collect();
    Ok((remaining, paragraphs))
}

pub fn in_section<'s, 'r, 'x>(context: Context<'r, 's>, section_name: &'x str) -> bool {
    for thing in context.iter() {
        match thing.get_data() {
            ContextElement::ExitMatcherNode(_) => {}
            ContextElement::PreviousElementNode(_) => {}
            ContextElement::Context(name) if *name == section_name => return true,
            ContextElement::Context(_) => {}
            ContextElement::StartOfParagraph => {} // TODO: If we specialize this to bold then this would be a good spot to stop scanning
        }
    }
    false
}
